@@ -1,4 +1,4 @@
-/* Copyright (C) 1999-2023 Free Software Foundation, Inc.
+/* Copyright (C) 2022-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    This program is free software; you can redistribute it and/or modify
@@ -66,7 +66,7 @@ glibc_hwcaps_subdirectory_name (const struct glibc_hwcaps_subdirectory *dir)
   return dir->name->string;
 }
 
-/* Linked list of known hwcaps subdirecty names.  */
+/* Linked list of known hwcaps subdirectory names.  */
 static struct glibc_hwcaps_subdirectory *hwcaps;
 
 struct glibc_hwcaps_subdirectory *
@@ -179,9 +179,6 @@ print_entry (const char *lib, int flag, uint64_t hwcap,
     case FLAG_SPARC_LIB64:
       fputs (",64bit", stdout);
       break;
-    case FLAG_IA64_LIB64:
-      fputs (",IA-64", stdout);
-      break;
     case FLAG_X8664_LIB64:
       fputs (",x86-64", stdout);
       break;
@@ -225,6 +222,12 @@ print_entry (const char *lib, int flag, uint64_t hwcap,
     case FLAG_RISCV_FLOAT_ABI_DOUBLE:
       fputs (",double-float", stdout);
       break;
+    case FLAG_LARCH_FLOAT_ABI_SOFT:
+      fputs (",soft-float", stdout);
+      break;
+    case FLAG_LARCH_FLOAT_ABI_DOUBLE:
+      fputs (",double-float", stdout);
+      break;
     case 0:
       break;
     default:
@@ -817,7 +820,7 @@ struct aux_cache_entry
   struct aux_cache_entry *next;
 };
 
-#define AUX_CACHEMAGIC		"glibc-ld.so.auxcache-1.0"
+#define AUX_CACHEMAGIC		"glibc-ld.so.auxcache-2.0"
 
 struct aux_cache_file_entry
 {
