@@ -1,5 +1,5 @@
 /* Get calling thread's ID.
-   Copyright (C) 2000-2022 Free Software Foundation, Inc.
+   Copyright (C) 2000-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -17,7 +17,7 @@
    <https://www.gnu.org/licenses/>.  */
 
 #include <pthread.h>
-
+#include <shlib-compat.h>
 #include <pt-internal.h>
 
 /* Return the thread ID of the calling thread.  */
@@ -25,9 +25,12 @@ pthread_t
 __pthread_self (void)
 {
   struct __pthread *self = _pthread_self ();
-  assert (self != NULL);
-
   return self->thread;
 }
 
-weak_alias (__pthread_self, pthread_self);
+libc_hidden_def (__pthread_self)
+versioned_symbol (libc, __pthread_self, pthread_self, GLIBC_2_21);
+
+#if OTHER_SHLIB_COMPAT (libpthread, GLIBC_2_12, GLIBC_2_21)
+compat_symbol (libc, __pthread_self, pthread_self, GLIBC_2_12);
+#endif
