@@ -1,7 +1,6 @@
 /* Mapping table for IBM1158.
-   Copyright (C) 2005-2019 Free Software Foundation, Inc.
+   Copyright (C) 2005-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Jiro SEKIBA <sekiba@jp.ibm.com>, 2005.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -15,7 +14,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <stdint.h>
 
@@ -89,13 +88,13 @@ static const uint32_t to_ucs4[256] =
 
 static const struct gap from_idx[] =
 {
-  { start: 0x0000, end: 0x00a0, idx:     0 },
-  { start: 0x00ad, end: 0x00ad, idx:   -12 },
-  { start: 0x0401, end: 0x045f, idx:  -863 },
-  { start: 0x0490, end: 0x0491, idx:  -911 },
-  { start: 0x20ac, end: 0x20ac, idx: -8105 },
-  { start: 0x2116, end: 0x2116, idx: -8210 },
-  { start: 0xffff, end: 0xffff, idx:     0 }
+  { .start = 0x0000, .end = 0x00a0, .idx =     0 },
+  { .start = 0x00ad, .end = 0x00ad, .idx =   -12 },
+  { .start = 0x0401, .end = 0x045f, .idx =  -863 },
+  { .start = 0x0490, .end = 0x0491, .idx =  -911 },
+  { .start = 0x20ac, .end = 0x20ac, .idx = -8105 },
+  { .start = 0x2116, .end = 0x2116, .idx = -8210 },
+  { .start = 0xffff, .end = 0xffff, .idx =     0 }
 };
 
 static const char from_ucs4[] =
