@@ -1,5 +1,5 @@
 /* Tables for conversion from and to IBM9030.
-   Copyright (C) 2005-2023 Free Software Foundation, Inc.
+   Copyright (C) 2005-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -88,9 +88,9 @@ static const uint32_t to_ucs4[256] =
 
 static const struct gap from_idx[] =
 {
-  { start: 0x0000, end: 0x00ac, idx:     0 },
-  { start: 0x0e01, end: 0x0e5b, idx: -3412 },
-  { start: 0xffff, end: 0xffff, idx:     0 }
+  { .start = 0x0000, .end = 0x00ac, .idx =     0 },
+  { .start = 0x0e01, .end = 0x0e5b, .idx = -3412 },
+  { .start = 0xffff, .end = 0xffff, .idx =     0 }
 };
 
 static const char from_ucs4[] =
