@@ -1,5 +1,5 @@
 /* Convert an IPv6 scope ID from text to the internal representation.
-   Copyright (C) 2016-2023 Free Software Foundation, Inc.
+   Copyright (C) 2016-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -49,7 +49,7 @@ __inet6_scopeid_pton (const struct in6_addr *address, const char *scope,
       char *end;
       unsigned long long number
         = ____strtoull_l_internal (scope, &end, /*base */ 10, /* group */ 0,
-                                   _nl_C_locobj_ptr);
+                                   /* bin_cst */ false, _nl_C_locobj_ptr);
       if (*end == '\0' && number <= UINT32_MAX)
         {
           *result = number;
