@@ -1,7 +1,6 @@
 /* Test setvbuf on readonly fopen (using mmap stdio).
-   Copyright (C) 2002-2021 Free Software Foundation, Inc.
+   Copyright (C) 2002-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Jakub Jelinek <jakub@redhat.com>, 2002.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -21,13 +20,17 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <libc-diag.h>
 
 int main (void)
 {
   char name[] = "/tmp/tst-mmap-setvbuf.XXXXXX";
   char buf[4096];
   const char * const test = "Let's see if mmap stdio works with setvbuf.\n";
+  DIAG_PUSH_NEEDS_COMMENT_CLANG;
+  DIAG_IGNORE_NEEDS_COMMENT_CLANG (3.4, "-Wgnu-folding-constant");
   char temp[strlen (test) + 1];
+  DIAG_POP_NEEDS_COMMENT_CLANG;
   int fd = mkstemp (name);
   FILE *f;
 
