@@ -1,5 +1,5 @@
 /* Test that _IO_wfile_sync does not crash (bug 20568).
-   Copyright (C) 2019-2021 Free Software Foundation, Inc.
+   Copyright (C) 2019-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -16,22 +16,43 @@
    License along with the GNU C Library; if not, see
    <https://www.gnu.org/licenses/>.  */
 
+#include <fcntl.h>
 #include <locale.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <wchar.h>
 #include <support/check.h>
+#include <support/xstdio.h>
 #include <support/xunistd.h>
+#include <support/temp_file.h>
+
+static const char test_data[] = "This is a test of _IO_wfile_sync.";
 
 static int
 do_test (void)
 {
+  static char *infile;
+  int infd;
+  FILE *infp;
+
+  infd = create_temp_file ("tst-wfile-sync-in-", &infile);
+  xwrite (infd, test_data, strlen (test_data));
+  xclose (infd);
+
+  infd = xopen (infile, O_RDONLY, 0);
+  infp = fdopen (infd, "r");
+
   TEST_VERIFY_EXIT (setlocale (LC_ALL, "de_DE.UTF-8") != NULL);
   /* Fill the stdio buffer and advance the read pointer.  */
-  TEST_VERIFY_EXIT (fgetwc (stdin) != WEOF);
+  TEST_VERIFY_EXIT (fgetwc (infp) != WEOF);
   /* This calls _IO_wfile_sync, it should not crash.  */
-  TEST_VERIFY_EXIT (setvbuf (stdin, NULL, _IONBF, 0) == 0);
+  TEST_VERIFY_EXIT (setvbuf (infp, NULL, _IONBF, 0) == 0);
   /* Verify that the external file offset has been synchronized.  */
-  TEST_COMPARE (xlseek (0, 0, SEEK_CUR), 1);
+  TEST_COMPARE (xlseek (infd, 0, SEEK_CUR), 1);
+
+  fclose (infp);
+  free (infile);
 
   return 0;
 }
