@@ -1,5 +1,5 @@
 /* Test for dynamic arrays.
-   Copyright (C) 2017-2023 Free Software Foundation, Inc.
+   Copyright (C) 2017-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -482,7 +482,7 @@ test_long_overflow (void)
        iteration would invoke undefined behavior.  That loop iteration
        can never be executed because an allocation of this size must
        fail.  */
-    DIAG_IGNORE_NEEDS_COMMENT (12, "-Waggressive-loop-optimizations");
+    DIAG_IGNORE_NEEDS_COMMENT_GCC (12, "-Waggressive-loop-optimizations");
     TEST_VERIFY (!dynarray_long_resize
                  (&dyn, (SIZE_MAX / sizeof (long)) + 1));
     DIAG_POP_NEEDS_COMMENT;
@@ -499,7 +499,7 @@ test_long_overflow (void)
        iteration would invoke undefined behavior.  That loop iteration
        can never be executed because an allocation of this size must
        fail.  */
-    DIAG_IGNORE_NEEDS_COMMENT (12, "-Waggressive-loop-optimizations");
+    DIAG_IGNORE_NEEDS_COMMENT_GCC (12, "-Waggressive-loop-optimizations");
     TEST_VERIFY (!dynarray_long_noscratch_resize
                  (&dyn, (SIZE_MAX / sizeof (long)) + 1));
     DIAG_POP_NEEDS_COMMENT;
