@@ -1,7 +1,6 @@
 /* Return maximum numeric value of X and Y.
-   Copyright (C) 1997-2021 Free Software Foundation, Inc.
+   Copyright (C) 1997-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Ulrich Drepper <drepper@cygnus.com>, 1997.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -22,6 +21,9 @@
 FLOAT
 M_DECL_FUNC (__fmax) (FLOAT x, FLOAT y)
 {
+#if M_USE_BUILTIN (FMAX)
+  return M_SUF (__builtin_fmax) (x, y);
+#else
   if (isgreaterequal (x, y))
     return x;
   else if (isless (x, y))
@@ -30,6 +32,7 @@ M_DECL_FUNC (__fmax) (FLOAT x, FLOAT y)
     return x + y;
   else
     return isnan (y) ? x : y;
+#endif
 }
 
 declare_mgen_alias (__fmax, fmax);
