@@ -1,4 +1,4 @@
-/* Copyright (C) 2012-2020 Free Software Foundation, Inc.
+/* Copyright (C) 2012-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -18,26 +18,47 @@
 #include <sys/auxv.h>
 #include <errno.h>
 #include <ldsodefs.h>
+#include <stdbool.h>
 
-
-unsigned long int
-__getauxval (unsigned long int type)
+bool
+__getauxval2 (unsigned long int type, unsigned long int *result)
 {
 #ifdef HAVE_AUX_VECTOR
   ElfW(auxv_t) *p;
 #endif
 
   if (type == AT_HWCAP)
-    return GLRO(dl_hwcap);
+    {
+      *result = GLRO(dl_hwcap);
+      return true;
+    }
   else if (type == AT_HWCAP2)
-    return GLRO(dl_hwcap2);
+    {
+      *result = GLRO(dl_hwcap2);
+      return true;
+    }
 
 #ifdef HAVE_AUX_VECTOR
   for (p = GLRO(dl_auxv); p->a_type != AT_NULL; p++)
     if (p->a_type == type)
-      return p->a_un.a_val;
+      {
+        *result = p->a_un.a_val;
+        return true;
+      }
 #endif
 
+  return false;
+}
+libc_hidden_def (__getauxval2)
+
+unsigned long int
+__getauxval (unsigned long int type)
+{
+  unsigned long int result;
+
+  if (__getauxval2 (type, &result))
+    return result;
+
   __set_errno (ENOENT);
   return 0;
 }
