@@ -1,4 +1,4 @@
-/* Copyright (C) 1992, 1995, 1996, 1997 Free Software Foundation, Inc.
+/* Copyright (C) 1992-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -13,21 +13,21 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <errno.h>
 #include <unistd.h>
 #include <sys/reboot.h>
+#include <libc-diag.h>
 
 /* Reboot the system.  */
 int
-reboot (howto)
-     int howto;
+reboot (int howto)
 {
+  ignore_value (howto);
+
   __set_errno (ENOSYS);
   return -1;
 }
 
-
 stub_warning (reboot)
-#include <stub-tag.h>
