@@ -1,7 +1,6 @@
 /* Get priority ceiling setting from pthread_mutexattr_t.
-   Copyright (C) 2006-2021 Free Software Foundation, Inc.
+   Copyright (C) 2006-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Jakub Jelinek <jakub@redhat.com>, 2006.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -19,11 +18,11 @@
 
 #include <pthreadP.h>
 #include <atomic.h>
-
+#include <shlib-compat.h>
 
 int
-pthread_mutexattr_getprioceiling (const pthread_mutexattr_t *attr,
-				  int *prioceiling)
+__pthread_mutexattr_getprioceiling (const pthread_mutexattr_t *attr,
+				    int *prioceiling)
 {
   const struct pthread_mutexattr *iattr;
   int ceiling;
@@ -46,3 +45,10 @@ pthread_mutexattr_getprioceiling (const pthread_mutexattr_t *attr,
 
   return 0;
 }
+versioned_symbol (libc, __pthread_mutexattr_getprioceiling,
+		  pthread_mutexattr_getprioceiling, GLIBC_2_34);
+
+#if OTHER_SHLIB_COMPAT (libpthread, GLIBC_2_4, GLIBC_2_34)
+compat_symbol (libpthread, __pthread_mutexattr_getprioceiling,
+               pthread_mutexattr_getprioceiling, GLIBC_2_4);
+#endif
