@@ -1,5 +1,5 @@
 /* Test glob compat symbol which avoid call GLOB_ALTDIRFUNC/gl_lstat.
-   Copyright (C) 2017-2021 Free Software Foundation, Inc.
+   Copyright (C) 2017-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -43,7 +43,7 @@ compat_symbol_reference (libc, glob, glob, GLIBC_2_0);
 #endif
 
 /* Compat glob should not call gl_lstat since for some old binaries it
-   might be unitialized (for instance GNUmake).  Check if it is indeed
+   might be uninitialized (for instance GNUmake).  Check if it is indeed
    not called.  */
 static bool stat_called;
 static bool lstat_called;
@@ -173,7 +173,8 @@ my_readdir (void *gdir)
 
   dir->d.d_type = filesystem[dir->idx].type;
 
-  strcpy (dir->d.d_name, filesystem[dir->idx].name);
+  __builtin___strcpy_chk (dir->d.d_name, filesystem[dir->idx].name,
+			  NAME_MAX);
 
   ++dir->idx;
 
