@@ -1,5 +1,5 @@
 /* Common f{truncate} tests definitions.
-   Copyright (C) 2016-2025 Free Software Foundation, Inc.
+   Copyright (C) 2016-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -36,7 +36,7 @@ static int temp_fd;
 static void
 do_prepare (void)
 {
-  temp_fd = create_temp_file ("tst-trucate.", &temp_filename);
+  temp_fd = create_temp_file ("tst-truncate.", &temp_filename);
   if (temp_fd == -1)
     {
       printf ("cannot create temporary file: %m\n");
