@@ -1,6 +1,6 @@
 /* Check if realpath does not consume extra stack space based on symlink
-   existance in the path (BZ #26341)
-   Copyright (C) 2021 Free Software Foundation, Inc.
+   existence in the path (BZ #26341)
+   Copyright (C) 2021-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -23,7 +23,7 @@
 #include <unistd.h>
 
 #define __sysconf sysconf
-#include <eloop-threshold.h>
+#include <min-eloop-threshold.h>
 #include <support/check.h>
 #include <support/support.h>
 #include <support/temp_file.h>
@@ -45,10 +45,16 @@ create_link (void)
   TEST_VERIFY_EXIT (fd != -1);
   xclose (fd);
 
+  /* Make filename a canonical path.  */
+  char *saved_filename = filename;
+  filename = realpath (filename, NULL);
+  free (saved_filename);
+  TEST_VERIFY (filename != NULL);
+
   /* Create MAXLINKS symbolic links to the temporary filename.
      On exit, linkname has the last link created.  */
   char *prevlink = filename;
-  int maxlinks = __eloop_threshold ();
+  int maxlinks = MIN_ELOOP_THRESHOLD;
   for (int i = 0; i < maxlinks; i++)
     {
       linkname = xasprintf ("%s%d", filename, i);
