@@ -1,4 +1,4 @@
-/* Copyright (C) 1999-2014 Free Software Foundation, Inc.
+/* Copyright (C) 1999-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -13,20 +13,21 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
+#include <libc-diag.h>
+#include <support/check.h>
 
 #define VAR "FOOBAR"
 
 char putenv_val[100] = VAR "=some longer value";
 
-int
-main (void)
+static int
+do_test (void)
 {
   int result = 0;
   const char *valp;
@@ -50,11 +51,7 @@ main (void)
 
   /* Getting this value should now be possible.  */
   valp = getenv (VAR);
-  if (valp == NULL || strcmp (valp, "one") != 0)
-    {
-      puts ("getenv #2 failed");
-      result = 1;
-    }
+  TEST_COMPARE_STRING (valp, "one");
 
   /* Try to replace without the replace flag set.  This should fail.  */
   if (setenv (VAR, "two", 0) != 0)
@@ -65,11 +62,7 @@ main (void)
 
   /* The value shouldn't have changed.  */
   valp = getenv (VAR);
-  if (valp == NULL || strcmp (valp, "one") != 0)
-    {
-      puts ("getenv #3 failed");
-      result = 1;
-    }
+  TEST_COMPARE_STRING (valp, "one");
 
   /* Now replace the value using putenv.  */
   if (putenv (putenv_val) != 0)
@@ -196,12 +189,17 @@ main (void)
       result = 1;
     }
 
+  /* This deliberately tests supplying a null pointer to a function whose
+     argument is marked __attribute__ ((nonnull)). */
+  DIAG_PUSH_NEEDS_COMMENT;
+  DIAG_IGNORE_NEEDS_COMMENT(5, "-Wnonnull");
   errno = 0;
   if (unsetenv (NULL) >= 0 || errno != EINVAL)
     {
       puts ("unsetenv #1 failed");
       result = 1;
     }
+  DIAG_POP_NEEDS_COMMENT;
 
   errno = 0;
   if (unsetenv ("") >= 0 || errno != EINVAL)
@@ -219,3 +217,6 @@ main (void)
 
   return result;
 }
+
+#define TEST_FUNCTION do_test ()
+#include "../test-skeleton.c"
