@@ -1,7 +1,6 @@
 /* Test memset functions.
-   Copyright (C) 1999-2021 Free Software Foundation, Inc.
+   Copyright (C) 1999-2026 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Written by Jakub Jelinek <jakub@redhat.com>, 1999.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -26,7 +25,11 @@
 # endif
 #else
 # ifndef WIDE
-#  define TEST_NAME "memset"
+#  ifdef TEST_MEMSET_EXPLICIT
+#   define TEST_NAME "memset_explicit"
+#  else
+#   define TEST_NAME "memset"
+#  endif
 # else
 #  define TEST_NAME "wmemset"
 # endif /* WIDE */
@@ -35,7 +38,11 @@
 #include "test-string.h"
 
 #ifndef WIDE
-# define MEMSET memset
+# ifdef TEST_MEMSET_EXPLICIT
+#  define MEMSET memset_explicit
+# else
+#  define MEMSET memset
+# endif
 # define CHAR char
 # define UCHAR unsigned char
 # define SIMPLE_MEMSET simple_memset
@@ -51,53 +58,21 @@
 # define BIG_CHAR WCHAR_MAX
 #endif /* WIDE */
 
-CHAR *SIMPLE_MEMSET (CHAR *, int, size_t);
-
 #ifdef TEST_BZERO
 typedef void (*proto_t) (char *, size_t);
-void simple_bzero (char *, size_t);
-void builtin_bzero (char *, size_t);
-
-IMPL (simple_bzero, 0)
-IMPL (builtin_bzero, 0)
-#ifdef TEST_EXPLICIT_BZERO
+# ifdef TEST_EXPLICIT_BZERO
 IMPL (explicit_bzero, 1)
-#else
+# else
 IMPL (bzero, 1)
-#endif
-
-void
-simple_bzero (char *s, size_t n)
-{
-  SIMPLE_MEMSET (s, 0, n);
-}
-
-void
-builtin_bzero (char *s, size_t n)
-{
-  __builtin_bzero (s, n);
-}
+# endif
 #else
 typedef CHAR *(*proto_t) (CHAR *, int, size_t);
-
-IMPL (SIMPLE_MEMSET, 0)
-# ifndef WIDE
-char *builtin_memset (char *, int, size_t);
-IMPL (builtin_memset, 0)
-# endif /* !WIDE */
 IMPL (MEMSET, 1)
-
-# ifndef WIDE
-char *
-builtin_memset (char *s, int c, size_t n)
-{
-  return __builtin_memset (s, c, n);
-}
-# endif /* !WIDE */
 #endif /* !TEST_BZERO */
 
+/* Naive implementation to verify results.  */
 CHAR *
-inhibit_loop_to_libcall
+test_cc_inhibit_loop_to_libcall
 SIMPLE_MEMSET (CHAR *s, int c, size_t n)
 {
   CHAR *r = s, *end = s + n;
@@ -107,18 +82,28 @@ SIMPLE_MEMSET (CHAR *s, int c, size_t n)
 }
 
 static void
-do_one_test (impl_t *impl, CHAR *s, int c __attribute ((unused)), size_t n)
+do_one_test (impl_t *impl, CHAR *s, int c __attribute ((unused)), size_t n, int space_below, int space_above)
 {
-  CHAR tstbuf[n];
+  CHAR buf[n];
+  CHAR sentinel = ~c;
+  if (space_below)
+      s[-1] = sentinel;
+  if (space_above)
+      s[n] = sentinel;
+  SIMPLE_MEMSET(s, ~c, n);
 #ifdef TEST_BZERO
-  simple_bzero (tstbuf, n);
+  SIMPLE_MEMSET (buf, 0, n);
   CALL (impl, s, n);
-  if (memcmp (s, tstbuf, n) != 0)
+  if (memcmp (s, buf, n) != 0
+      || (space_below && s[-1] != sentinel)
+      || (space_above && s[n] != sentinel))
 #else
   CHAR *res = CALL (impl, s, c, n);
   if (res != s
-      || SIMPLE_MEMSET (tstbuf, c, n) != tstbuf
-      || MEMCMP (s, tstbuf, n) != 0)
+      || SIMPLE_MEMSET (buf, c, n) != buf
+      || MEMCMP (s, buf, n) != 0
+      || (space_below && s[-1] != sentinel)
+      || (space_above && s[n] != sentinel))
 #endif /* !TEST_BZERO */
     {
       error (0, 0, "Wrong result in function %s", impl->name);
@@ -130,12 +115,16 @@ do_one_test (impl_t *impl, CHAR *s, int c __attribute ((unused)), size_t n)
 static void
 do_test (size_t align, int c, size_t len)
 {
-  align &= 7;
+  int space_below, space_above;
+  align &= 4095;
   if ((align + len) * sizeof (CHAR) > page_size)
     return;
 
+  space_below = !!align;
+  space_above = !((align + len + 1) * sizeof (CHAR) > page_size);
+
   FOR_EACH_IMPL (impl, 0)
-    do_one_test (impl, (CHAR *) (buf1) + align, c, len);
+    do_one_test (impl, (CHAR *) (buf1) + align, c, len, space_below, space_above);
 }
 
 #ifndef TEST_BZERO
@@ -245,9 +234,11 @@ test_main (void)
     {
       for (i = 0; i < 18; ++i)
 	do_test (0, c, 1 << i);
-      for (i = 1; i < 32; ++i)
+      for (i = 1; i < 64; ++i)
 	{
 	  do_test (i, c, i);
+	  do_test (4096 - i, c, i);
+	  do_test (4095, c, i);
 	  if (i & (i - 1))
 	    do_test (0, c, i);
 	}
