@@ -39,6 +39,7 @@
  */
 #include <stdio.h>
 #include <unistd.h>
+#include <stdint.h>
 #include <rpc/rpc.h>
 #include <rpc/clnt.h>
 #include <sys/types.h>
@@ -48,6 +49,7 @@
 #include <rpc/auth_des.h>
 #include <errno.h>
 #include <netinet/in.h>
+#include <shlib-compat.h>
 
 #define NYEARS	(u_long)(1970 - 1900)
 #define TOFFSET (u_long)(60*60*24*(365*NYEARS + (NYEARS/4)))
@@ -73,7 +75,7 @@ rtime (struct sockaddr_in *addrp, struct rpc_timeval *timep,
   int milliseconds;
   int res;
   /* RFC 868 says the time is transmitted as a 32-bit value.  */
-  uint32_t thetime;
+  uint32_t thetime = 0;
   struct sockaddr_in from;
   socklen_t fromlen;
   int type;
@@ -140,4 +142,4 @@ rtime (struct sockaddr_in *addrp, struct rpc_timeval *timep,
   timep->tv_usec = 0;
   return 0;
 }
-libc_hidden_def (rtime)
+libc_hidden_nolink_sunrpc (rtime, GLIBC_2_1)
