@@ -1,4 +1,4 @@
-/* Copyright (C) 1997-2023 Free Software Foundation, Inc.
+/* Copyright (C) 1997-2026 Free Software Foundation, Inc.
 
    This file is part of the GNU C Library.
 
@@ -26,5 +26,5 @@ __feholdexcept (fenv_t *envp)
   return 0;
 }
 libm_hidden_def (__feholdexcept)
-weak_alias (__feholdexcept, feholdexcept)
+static_weak_alias (__feholdexcept, feholdexcept)
 libm_hidden_weak (feholdexcept)
